from ftw.builder import create, Builder
from ftw.file.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import statusmessages


class TestProtectedFiles(FunctionalTestCase):

    def setUp(self):
        super(TestProtectedFiles, self).setUp()
        self.grant('Manager')

    @browsing
    def test_protected_file_cannot_be_deleted(self, browser):
        file = create(Builder('file').titled('My Image').attach_asset('testimage.jpg'))

        browser.login()

        # Mark the file as protected.
        browser.visit(file, view='edit')
        browser.fill({'Protected': True}).submit()

        # Click the "delete" action
        browser.find('Delete').click()

        # Submit the confirmation form.
        with browser.expect_http_error():
            browser.find('Delete').click()
        self.assertEqual(
            ['You are not allowed to delete the file "My Image".'],
            statusmessages.error_messages()
        )



from Acquisition import aq_inner, aq_parent
from plone.app.contentrules.handlers import execute


def downloaded(event):
    """When a file is downloaded, execute rules assigned to its parent
    """
    execute(aq_parent(aq_inner(event.object)), event)



from ftw.upgrade import UpgradeStep


class ChangePortalPropertiesInstallMissing(UpgradeStep):

    def __call__(self):
        self.setup_install_profile(
            'profile-ftw.file.upgrades:1511')

        self.setup_install_profile(
            'profile-ftw.calendarwidget:default')



from plone.app.uuid.utils import uuidToObject
from Products.TinyMCE.adapters.JSONFolderListing import JSONFolderListing
import json


class FtwJSONFolderListing(JSONFolderListing):

    def getListing(self, filter_portal_types, rooted, document_base_url,
                   upload_type=None, image_types=None):
        listing = super(FtwJSONFolderListing, self).getListing(
            filter_portal_types, rooted, document_base_url,
            upload_type=upload_type, image_types=image_types)
        if upload_type == 'Image':
            listing = self.remove_files_which_are_not_images(listing)
        return listing

    def remove_files_which_are_not_images(self, json_encoded_listing):
        """The super getListing call returns all ftw.file files, no matter
        if they are images or not.
        When we require images, we therefore need to filter the files so
        that we only return images.
        """

        listing_obj = json.loads(json_encoded_listing)

        filtered_items = []
        for item in listing_obj['items']:
            obj = uuidToObject(item['uid'])
            if hasattr(obj, 'is_image') and not obj.is_image():
                continue
            filtered_items.append(item)
        listing_obj['items'] = filtered_items

        return json.dumps(listing_obj)



from ZODB.POSException import ConflictError
from logging import exception
from plone.app.imaging.interfaces import IImageScaling, IImageScaleFactory
from plone.app.imaging.scaling import ImageScaling
from zope.component import queryAdapter
from zope.interface import alsoProvides
from zope.interface import implements
from zope.publisher.interfaces import IPublishTraverse
from zope.traversing.interfaces import ITraversable


class FtwImageScaling(ImageScaling):
    """ view used for generating (and storing) image scales """
    implements(IImageScaling, ITraversable, IPublishTraverse)

    def create(self, fieldname, direction='keep', **parameters):
        """ factory for image scales, see `IImageScaleStorage.scale` """
        field = self.field(fieldname)
        alsoProvides(field, IImageScaleFactory)
        create = queryAdapter(field, IImageScaleFactory).create
        try:
            return create(self.context, direction=direction, **parameters)
        except IOError:
            return None
        except OverflowError:
            # Some PSDs throw an OverflowError, probably because of unsupported
            # file formats.
            # https://github.com/4teamwork/ftw.file/issues/29
            return None
        except (ConflictError, KeyboardInterrupt):
            raise
        except Exception:
            if not getattr(field, 'swallowResizeExceptions', False):
                raise
            else:
                exception('could not scale "%r" of %r',
                          field, self.context.absolute_url())



import json
import transaction

from Acquisition import aq_inner
from Acquisition import aq_parent
from Acquisition import aq_base
from ftw.file.utils import is_image
from plone import api
from plone.outputfilters.browser.resolveuid import uuidFor
from plone.restapi.exceptions import DeserializationError
from plone.restapi.interfaces import IDeserializeFromJson
from Products.CMFCore.interfaces._content import IFolderish
from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage
from Products.TinyMCE.adapters.Upload import Upload
from zExceptions import BadRequest
from zope.component import queryMultiAdapter
from zope.i18n import translate
from zope.publisher.browser import BrowserView

from ftw.file import fileMessageFactory as _


class FileUpload(BrowserView):
    """View for handling drag-and-drop file replace"""

    def __call__(self):
        self.file = self.request.get('file')
        try:
            if not self.file:
                raise BadRequest('No content provided.')

            # Borrow code from plone.restapi which handles validation correctly
            deserializer = queryMultiAdapter((self.context, self.request), IDeserializeFromJson)
            if deserializer is None:
                self.request.response.setStatus(501)
                return json.dumps(dict(error=dict(
                    message="Cannot deserialize type {}".format(self.context.portal_type))
                ))

            # Will raise BadRequest if validation fails
            deserializer(data={'file': self.file, 'filename': self.file.filename})

            portal = api.portal.get()
            repository_tool = getToolByName(portal, 'portal_repository')

            if repository_tool.isVersionable(self.context):
                # TODO: This creates another entry in the history resulting
                # in two consecutive history entries.
                repository_tool.save(
                    self.context,
                    comment=translate(_('File replaced with Drag & Drop.'),
                                      context=self.request)
                )
        except BadRequest as br:
            transaction.doom()
            # Show validation failures on next page (file_view)
            if type(br.message) in (list, tuple):
                errors = br.message
                for msg in errors:
                    IStatusMessage(self.request).add(msg['message'], 'error')
            else:
                errors = [{'message': br.message}]
                IStatusMessage(self.request).add(br.message, 'error')

            self.request.response.setStatus(400)
            retval = {'success': False, 'errors': errors}
            self.request.response.setBody(retval, lock=True)
            return json.dumps(retval)
        else:
            # Note: deserializer has handled notify for ObjectEditedEvent
            return json.dumps({'success': True})


class TinyMCEFileUpload(Upload):

    def upload(self):
        """Adds uploaded file.

        Required params: uploadfile, uploadtitle, uploaddescription
        """
        context = aq_inner(self.context)
        self.request = context.REQUEST
        if not IFolderish.providedBy(context):
            context = aq_parent(context)

        request = context.REQUEST
        utility = getToolByName(context, 'portal_tinymce')

        id_ = request['uploadfile'].filename
        content_type = request['uploadfile'].headers["Content-Type"]
        # check if container is ready to store images
        if self.is_temporary(context):
            return self.errorMessage(
                translate(_('Please save the object first'
                            ' to enable image upload.'),
                          context=self.request))

        # check mime type to make sure an image is uploaded
        if not is_image(content_type):
            return self.errorMessage(
                translate(_('Only image upload allowed.'),
                          context=self.request))

        # Permission checks based on code by Danny Bloemendaal

        # 1) check if the current user has permissions to add stuff
        if not context.portal_membership.checkPermission(
            'Add portal content', context):
            return self.errorMessage(
                "You do not have permission to upload files in this folder")

        # 2) check image types uploadable in folder.
        #    priority is to content_type_registry image type
        allowed_types = [t.id for t in context.getAllowedTypes()]
        tiny_image_types = utility.imageobjects.split('\n')
        uploadable_types = []
        for typename in tiny_image_types:
            if typename in allowed_types:
                uploadable_types.append(typename)

        # Get an unused filename without path
        id_ = self.cleanupFilename(id_)

        for metatype in uploadable_types:
            try:
                newid = context.invokeFactory(type_name=metatype, id=id_)
                if newid is None or newid == '':
                    newid = id_
                break
            except ValueError:
                continue
            except BadRequest:
                return self.errorMessage(
                    translate(_("Bad filename, please rename."),
                              context=self.request))
        else:
            return self.errorMessage(translate(
                _("Not allowed to upload a file of this type to this folder"),
                context=self.request))

        obj = getattr(context, newid, None)
        if not obj:
            return self.errorMessage("Could not upload the file")

        # Update fields
        # Borrow code from plone.restapi which handles validation correctly
        try:
            deserializer = queryMultiAdapter((obj, self.request), IDeserializeFromJson)
            if deserializer is None:
                return self.errorMessage("Cannot deserialize type {}".format(obj.portal_type))

            try:
                uploaded_file = request['uploadfile']
                field_data = {
                    'file': uploaded_file,
                    'filename': uploaded_file.filename,
                    'title': request['uploadtitle'] or uploaded_file.filename,
                    'description': request['uploaddescription']
                }
                # Will raise BadRequest if validation fails
                # it also looks after notify(ObjectInitializedEvent ...)
                deserializer(data=field_data, validate_all=True, create=True)
            except DeserializationError as e:
                return self.errorMessage("DeserializationError: {}".format(str(e)))
        except BadRequest as br:
            transaction.doom()
            if type(br.message) in (list, tuple):
                # Multiple errors are so unlikely (and hard to test) that we
                # just return the first error
                error = br.message[0]['message']
            else:
                error = br.message
            return self.errorMessage(error)

        obj.reindexObject()
        folder = obj.aq_parent.absolute_url()

        if utility.link_using_uids:
            path = "resolveuid/%s" % (uuidFor(obj))
        else:
            path = obj.absolute_url()
        return self.okMessage(path, folder)

    def is_temporary(self, obj, checkId=True):
        """Checks, whether an object is a temporary object (means it's in the
        `portal_factory`) or has no acquisition chain set up.
        Source: http://svn.plone.org/svn/collective/collective.indexing
        /trunk/collective/indexing/subscribers.py
        """
        parent = aq_parent(aq_inner(obj))
        if parent is None:
            return True
        if checkId and getattr(obj, 'getId', None):
            if getattr(aq_base(parent), obj.getId(), None) is None:
                return True
        isTemporary = getattr(obj, 'isTemporary', None)
        if isTemporary is not None:
            try:
                if obj.isTemporary():
                    return True
            except TypeError:
                # `isTemporary` on the `FactoryTool` expects 2 args
                return True
        return False



from unittest import TestCase
from ftw.file.testing import FTW_FILE_INTEGRATION_TESTING
from StringIO import StringIO
from plone.app.testing import login
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
import os


class TestIsImage(TestCase):

    layer = FTW_FILE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

    def test_isimage(self):
        file_data = open("%s/assets/transparent.gif" % os.path.split(__file__)[0], 'r')
        file_ = self.portal.get(self.portal.invokeFactory('File', 'myfile', file=file_data))
        self.assertTrue(file_.is_image())

    def test_isnotimage(self):
        file_data = StringIO('blubber blubb')
        file_ = self.portal.get(self.portal.invokeFactory('File', 'myfile', file=file_data))
        self.assertFalse(file_.is_image())

    def test_ioerror(self):
        """Check if IOError isn't raised when PIL can't open the file.
            This applies to filetype where we only need libaries in some cases
            and broken images.
        """
        file_ = open("%s/assets/CCITT_1.TIF" % os.path.split(__file__)[0], 'r')
        file_ = self.portal.get(self.portal.invokeFactory('File', 'myfile', file=file_))
        self.assertTrue(file_.is_image())
        view = file_.restrictedTraverse('@@file_view')
        self.assertFalse(view.get_image_tag())



from collective.clamav.testing import AVMOCK_AT_FIXTURE
from ftw.builder.session import BuilderSession
from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import set_builder_session_factory
from ftw.testing.layer import COMPONENT_REGISTRY_ISOLATION
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.testing.z2 import installProduct
from zope.component import eventtesting
from zope.configuration import xmlconfig
import ftw.file.tests.builders  # noqa


def functional_session_factory():
    sess = BuilderSession()
    sess.auto_commit = True
    return sess


class FtwFileLayer(PloneSandboxLayer):

    defaultBases = (COMPONENT_REGISTRY_ISOLATION, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '</configure>',
            context=configurationContext)

        installProduct(app, 'ftw.file')

    def setUpPloneSite(self, portal):
        # Install into Plone site using portal_setup
        applyProfile(portal, 'plone.app.imaging:default')
        applyProfile(portal, 'plone.app.registry:default')
        applyProfile(portal, 'ftw.file:default')
        eventtesting.setUp()

FTW_FILE_FIXTURE = FtwFileLayer()
FTW_FILE_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FTW_FILE_FIXTURE, ), name="ftw.file:Integration")

FTW_FILE_AV_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FTW_FILE_FIXTURE, AVMOCK_AT_FIXTURE ),
    name="ftw.file:AntiVirusIntegration")

FTW_FILE_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FTW_FILE_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="ftw.file:Functional")

FTW_FILE_AV_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FTW_FILE_FIXTURE,
           AVMOCK_AT_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="ftw.file:AntiVirusFunctional")




from ftw.upgrade import UpgradeStep


class ReindexIcons(UpgradeStep):

    def __call__(self):
        cat = self.portal.portal_catalog
        all_values = cat.uniqueValuesFor('getContentType')
        values = []
        for value in all_values:
            if value.startswith('image'):
                values.append(value)
        values.append('application/pdf')
        query = {'portal_type': 'File', 'getContentType': values}
        for obj in self.objects(query, 'update getIcon metadata'):
            cat.reindexObject(obj, idxs=['Title'], update_metadata=True)



from collective.clamav.interfaces import IAVScannerSettings
from collective.clamav.testing import EICAR
from collective.clamav.validator import SCAN_RESULT_KEY
from ftw.builder import create
from ftw.builder import Builder
from ftw.file.testing import FTW_FILE_AV_FUNCTIONAL_TESTING
from ftw.file.testing import FTW_FILE_AV_INTEGRATION_TESTING
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import statusmessages
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.registry.interfaces import IRegistry
from six import StringIO
from unittest import TestCase
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility

import json
import transaction


class TestFileVirusScanningDownloads(TestCase):

    layer = FTW_FILE_AV_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        # Don't scan files while creating test fixtures
        registry = getUtility(IRegistry)
        registry.forInterface(IAVScannerSettings).clamav_enabled = False
        transaction.commit()
        self.context = create(Builder('file').attach_file_containing(
                              EICAR, 'contaminated.txt'), processForm=False)
        registry.forInterface(IAVScannerSettings).clamav_enabled = True
        transaction.commit()

        # Make sure the above doesn't cache a scan attempt in the request
        # and ruin our test
        annotations = IAnnotations(self.portal.REQUEST)
        if SCAN_RESULT_KEY in annotations:
            del annotations[SCAN_RESULT_KEY]

    @browsing
    def test_prevent_download_of_virus_file(self, browser):
        """
        Test we prevent the download of a file with virus (unless virus scanning is explicitly off)
        """
        browser.login().visit(self.context, view='@@download')
        statusmessages.assert_message(
            'Download blocked. The malware Eicar-Test-Signature has been found in the file.'
        )


class TestVirusScanningUploads(TestCase):

    layer = FTW_FILE_AV_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        # Don't scan files while creating test fixtures
        registry = getUtility(IRegistry)
        registry.forInterface(IAVScannerSettings).clamav_enabled = False
        transaction.commit()
        self.file_ = create(
            Builder('file')
                .titled('Document')
                .attach_file_containing('My PDF', name='test.pdf')
        )
        registry.forInterface(IAVScannerSettings).clamav_enabled = True
        transaction.commit()
        # Make sure the above doesn't cache a scan attempt in the request
        # and ruin our test
        annotations = IAnnotations(self.portal.REQUEST)
        if SCAN_RESULT_KEY in annotations:
            del annotations[SCAN_RESULT_KEY]


    def test_virus_protection_drag_and_drop(self):
        """ Clamav validator should stop updates with a virus infected file """
        dodgy_file = StringIO(EICAR)
        dodgy_file.filename = 'innocent_looking_file.txt'
        dodgy_file.content_type = 'text/plain'
        self.portal.REQUEST.set('file', dodgy_file)

        ajax_upload_view = self.file_.restrictedTraverse('ajax-upload')
        response = json.loads(ajax_upload_view())

        self.assertFalse(response['success'])
        self.assertEqual('Validation failed, file is virus-infected. (Eicar-Test-Signature FOUND)',
                         response['errors'][0]['message'])



from ftw.upgrade import UpgradeStep


class SetEmptyDocumentDate(UpgradeStep):
    """Set empty document date.
    """

    def __call__(self):
        self.install_upgrade_profile()
        query = {'object_provides': 'ftw.file.interfaces.IFile'}
        for obj in self.objects(query, 'Update empty documentDate'):

            if obj.getDocumentDate():
                # We skip files with a documentdate
                continue

            obj.setDocumentDate(obj.created())
            obj.reindexObject(idxs=['documentDate'])



from plone.outputfilters.filters import resolveuid_and_caption
from plone.app.imaging.interfaces import IImageScale


class FtwFileFilter(resolveuid_and_caption.ResolveUIDAndCaptionFilter):

    def resolve_image(self, src):
        (image, fullimage, source, description) = resolveuid_and_caption.\
            ResolveUIDAndCaptionFilter.resolve_image(self, src)
        if not IImageScale.providedBy(image):
            url = ''
            obj, subpath, appendix = self.resolve_link(src)
            if image:
                url = image.absolute_url()
            if isinstance(url, unicode):
                url = url.encode('utf8')
            src = '/'.join([url, subpath, appendix])
        else:
            src = image.absolute_url()
        return image, fullimage, src, description



from ftw.file.testing import FTW_FILE_FUNCTIONAL_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.testing.z2 import Browser
from StringIO import StringIO
from unittest import TestCase
import transaction


class TestFileOverlay(TestCase):

    layer = FTW_FILE_FUNCTIONAL_TESTING

    def setUp(self):
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False

        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.file_ = StringIO(
                    'GIF89a\x01\x00\x01\x00\x80\x00\x00\x00\x00\x00\x00\x00'
                    '\x00!\xf9\x04\x04\x00\x00\x00\x00,\x00\x00\x00\x00\x01'
                    '\x00\x01\x00\x00\x02\x02D\x01\x00;')

        self.portal.invokeFactory('File', 'f1', file=self.file_)
        self.context = self.portal.f1
        transaction.commit()

    def login(self):
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (TEST_USER_NAME, TEST_USER_PASSWORD,))

    def test_image_displayed(self):
        self.browser.open(self.context.absolute_url() + '/view')
        self.assertIn('<a id="preview" class="colorboxLink"'
                      ' href="http://nohost/plone/f1/@@images/file">',
                      self.browser.contents)
        self.assertIn('<img src="http://nohost/plone/f1/@@images',
                      self.browser.contents)

    def test_image_link_works(self):
        self.browser.open(self.context.absolute_url() + '/@@images/file')
        self.assertEqual('image/gif', self.browser.headers['content-type'])
        self.assertEqual(self.file_.read(), self.browser.contents)



from ftw.builder import Builder
from ftw.builder import create
from ftw.file.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from unittest import skipUnless
import pkg_resources


try:
    pkg_resources.require('ftw.activity>=2')
except (pkg_resources.VersionConflict, pkg_resources.DistributionNotFound):
    HAS_ACTIVITY_2 = False
else:
    HAS_ACTIVITY_2 = True
    from ftw.activity.tests.helpers import get_soup_activities


@skipUnless(HAS_ACTIVITY_2, 'ftw.activity 2.0 not installed')
class TestActivity2Downloads(FunctionalTestCase):

    @browsing
    def test_activity_created_when_file_downloaded(self, browser):
        self.grant('Contributor')
        file_ = create(Builder('file').with_dummy_content())
        browser.login().open(file_, view='@@download')

        self.assertIn(
            {'path': '/plone/file',
             'action': 'file:downloaded'},
            get_soup_activities(('path', 'action')))



import json
from StringIO import StringIO
from Products.ATContentTypes.interfaces import IATFile
from Products.CMFCore.utils import getToolByName

from ftw.builder import Builder
from ftw.builder import create
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest import TestCase

from ftw.file.testing import FTW_FILE_INTEGRATION_TESTING


class TestAjaxUpload(TestCase):

    layer = FTW_FILE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.file_ = create(
            Builder('file')
                .titled('Document')
                .attach_file_containing('My PDF', name='test.pdf')
        )

        self.new_file = StringIO('Raw file data')
        self.new_file.filename = 'new_file.txt'
        self.new_file.content_type = 'text/plain'

    def test_replace_file(self):
        self.portal.REQUEST.set('file', self.new_file)

        ajax_upload_view = self.file_.restrictedTraverse('ajax-upload')
        response = json.loads(ajax_upload_view())

        self.assertEqual(response['success'], True)
        replaced_file = IATFile(self.portal['document']).getFile()
        self.assertEqual(replaced_file.filename, 'new_file.txt')
        self.assertEqual(replaced_file.data, 'Raw file data')

    def test_empty_file(self):
        """ Archetypes validator should stop updates with an empty file """
        empty_file = StringIO('')
        empty_file.filename = 'empty_file.txt'
        empty_file.content_type = 'text/plain'
        self.portal.REQUEST.set('file', empty_file)

        ajax_upload_view = self.file_.restrictedTraverse('ajax-upload')
        response = json.loads(ajax_upload_view())

        self.assertFalse(response['success'])
        self.assertIn('Uploaded file is empty',
                      response['errors'][0]['message'])

    def test_no_file(self):
        ajax_upload_view = self.file_.restrictedTraverse('ajax-upload')
        response = json.loads(ajax_upload_view())

        self.assertFalse(response['success'])
        self.assertEqual('No content provided.',
                         response['errors'][0]['message'])

    def test_new_version(self):
        portal = api.portal.get()
        repository_tool = getToolByName(portal, 'portal_repository')
        repository_tool.setVersionableContentTypes('File')

        self.portal.REQUEST.set('file', self.new_file)
        self.file_.restrictedTraverse('ajax-upload')()

        history = repository_tool.getHistory(self.file_)
        self.assertEqual(1, len(history))
        replaced_file = IATFile(self.portal['document']).getFile()
        self.assertEqual(replaced_file.filename, 'new_file.txt')
        self.assertEqual(replaced_file.data, 'Raw file data')



from ftw.activity.catalog import get_activity_soup
from ftw.activity.catalog.record import ActivityRecord


def make_file_downloaded_activity(context, event):
    file_downloaded(context)


def file_downloaded(context, actor_userid=None, date=None):
    record = ActivityRecord(context, 'file:downloaded',
                            actor_userid=actor_userid, date=date)
    return get_activity_soup().add(record)



from DateTime import DateTime
from ftw.upgrade import ProgressLogger
from ftw.upgrade import UpgradeStep
from zope.annotation import IAnnotations


ANN_KEY = 'ftw.file-upgrade-1500-migrated-data'


class AddDocumentDateIndex(UpgradeStep):

    def __call__(self):
        self.setup_install_profile('profile-ftw.file.upgrades:1500')
        if not self.catalog_has_index('documentDate'):
            self.catalog_add_index('documentDate', 'DateIndex')
        self.migrate_dates()
        self.catalog_reindex_objects({'portal_type': 'File'},
                                     idxs=['effective', 'documentDate'])

    def migrate_dates(self):
        objects = self.catalog_unrestricted_search({'portal_type': 'File'},
                                                   full_objects=True)

        with ProgressLogger('Migrate file dates', objects) as step:
            for obj in objects:
                ann = IAnnotations(obj)
                if ann.get(ANN_KEY, None) is not None:
                    step()
                    continue

                ann[ANN_KEY] = True

                newdate = obj.getEffectiveDate()
                if not newdate:
                    newdate = obj.created()
                obj.setDocumentDate(newdate)
                obj.setEffectiveDate(DateTime())
                step()



from ftw.builder import Builder
from ftw.builder import create
from ftw.file.testing import FTW_FILE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from Products.Archetypes.interfaces.base import IBaseObject
from Products.Five import BrowserView
from unittest import TestCase
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class TestView(BrowserView):

    def __call__(self):
        return "Success"


class TestViewnameEqualsScale(TestCase):

    layer = FTW_FILE_FUNCTIONAL_TESTING

    def setUp(self):
        gsm = self.layer['portal'].getSiteManager()
        setRoles(self.layer['portal'], TEST_USER_ID, ['Manager'])

        gsm.registerAdapter(TestView,
                            (IBaseObject,IDefaultBrowserLayer),
                            Interface,
                            name="test_mini")
        self.file = create(Builder('file').attach_asset("transparent.gif"))

    @browsing
    def test_view_success(self, browser):
        page = browser.login().visit(self.file, view="test_mini")
        self.assertEquals("Success", page.contents)



from ftw.file.testing import FTW_FILE_INTEGRATION_TESTING
from Products.CMFCore.utils import getToolByName
from unittest import TestCase


class TestCatalog(TestCase):
    layer = FTW_FILE_INTEGRATION_TESTING

    def setUp(self):
        self.catalog = getToolByName(self.layer['portal'], 'portal_catalog')

    def test_getCategoryUids_index_registered(self):
        self.assertIn('documentDate', self.catalog.indexes())

    def test_getCategoryUids_metadata_registered(self):
        self.assertIn('documentDate', self.catalog.schema())



from ftw.activity.browser.representations import DefaultRepresentation
from ftw.file.interfaces import IFile
from ftw.file.utils import FileMetadata
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import adapts
from zope.interface import Interface


class FileRepresentation(DefaultRepresentation):
    adapts(IFile, Interface)
    index = ViewPageTemplateFile('activity.pt')

    def get_image_tag(self):
        return FileMetadata(self.context).get_image_tag(
            fieldname='file', height=100)



from ftw.upgrade import UpgradeStep


class AllowSiteAdministratorAddFiles(UpgradeStep):
    """Allow site administrator add files.
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.file.config import INDEXES
from Products.CMFCore.utils import getToolByName
import logging


LOGGER = logging.getLogger('ftw.file')


def installed(site):
    add_indexes(site)


def add_indexes(context):
    """Add our indexes to the catalog.

    Doing it here instead of in profiles/default/catalog.xml means we
    do not need to reindex those indexes after every reinstall.
    """

    catalog = getToolByName(context, 'portal_catalog')
    indexes = catalog.indexes()

    for name, meta_type in INDEXES:
        if name not in indexes:
            catalog.addIndex(name, meta_type)
            LOGGER.info('Added index: %s' % name)



from ftw.upgrade import UpgradeStep
from Products.CMFPlone.utils import getToolByName


class UpdateContenttype(UpgradeStep):

    def __call__(self):
        mtr = getToolByName(self.portal, 'mimetypes_registry')
        cat = getToolByName(self.portal, 'portal_catalog')
        contenttypes = ['application/msword', 'application/octet-stream']
        query = {'portal_type': 'File', 'getContentType': contenttypes}
        for obj in self.objects(query, 'update Contenttype'):
            file_ = obj.getField('file').get(obj)
            mimetype = mtr.classify(file_.data, filename=file_.getFilename())
            if str(mimetype) != file_.getContentType():
                file_.setContentType(str(mimetype))
                cat.reindexObject(obj,
                                  idxs=['getContentType'],
                                  update_metadata=True
                                  )



from AccessControl import Unauthorized
from Products.ATContentTypes.browser.download import DownloadArchetypeFile
from zope.publisher.interfaces import NotFound
import urllib


class DownloadFileView(DownloadArchetypeFile):

    def __call__(self):
        if self.fieldname is not None:
            return self.call_download()

        field = self.context.getPrimaryField()
        content = field.get(self.context)
        if not content:
            return self.call_download()

        filename = content.filename
        if isinstance(filename, unicode):
            filename = filename.encode('utf-8')

        # We will use the filename in the url and having a percent sign (even
        # a quoted one) in the url seems to cause problems with apache
        # mod_rewrite and look-ahead variables. Thus remove it.
        filename = filename.replace('%', '')
        filename = urllib.quote(filename)

        download_url = '/'.join((
                self.context.absolute_url(),
                '@@download',
                field.getName(),
                filename))

        if 'inline' in self.request.form.keys():
            download_url += '?inline=true'

        return self.request.RESPONSE.redirect(download_url)

    def call_download(self):
        context = getattr(self.context, 'aq_explicit', self.context)
        field = context.getField(self.fieldname)
        if field is None:
            raise NotFound(self, self.fieldname, self.request)
        if not field.checkPermission('r', context):
            raise Unauthorized()

        disposition = 'attachment'
        if 'inline' in self.request.form.keys():
            disposition = 'inline'

        return field.index_html(context, disposition=disposition)

    def __contains__(self, item):
        """
        Make this browser view iterable in order to allow HEAD requests on
        files. This is needed because `ZPublisher.BaseRequest.BaseRequest#traverse`
        falls into WebDAV mode upon HEAD requests (tested against Zope2 2.13.24).
        """
        return item in [self.filename, self.fieldname]



from ftw.builder import Builder
from ftw.builder import create
from ftw.file.testing import FTW_FILE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest import TestCase
import transaction


class TestPagination(TestCase):
    layer = FTW_FILE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.layer['portal'], TEST_USER_ID, ['Contributor'])

        self.file = create(Builder('file')
                           .titled('An important file')
                           .with_dummy_content())

    @browsing
    def test_pagination_is_used(self, browser):
        # enable history
        portal_repository = self.portal.portal_repository
        portal_repository.addPolicyForContentType(u'File',
                                                  u'at_edit_autoversion')
        portal_repository.setVersionableContentTypes(
            portal_repository.getVersionableContentTypes() + [u'File'])
        transaction.commit()

        # make history entrys
        for i in range(12):
            browser.login().open(self.file, view='edit')
            browser.fill({'Title': str(i)}).submit()

        self.assertIn("file_download_version?version_id=11",
                      browser.contents,
                      "Revision id 11 is new and should be "
                      "displayed in pagination.")
        self.assertNotIn('file_download_version?version_id=2"',
                         browser.contents,
                         "Revision 0 is old and should be on the second page "
                         " of the pagination view.")

    @browsing
    def test_pagination_is_disabled_without_history(self, browser):
        browser.login().open(self.file)
        self.assertNotIn(
            'error while rendering plone.belowcontentbody.inlinecontenthistory',
            browser.contents,
            'Pagination should not fail if there is no history.')



from ftw.file import fileMessageFactory as _
from plone import api
from Products.CMFPlone.utils import safe_unicode
from zope.i18n import translate


def handle_protected_file(obj, event):
    if obj.getIsProtected():
        api.portal.show_message(
            message=translate(
                _(
                    u'message_protected_file',
                    default=u'You are not allowed to delete the file "${file_title}".',
                    mapping={
                        'file_title': safe_unicode(obj.Title()),
                    },
                ),
                context=obj.REQUEST,
            ),
            request=obj.REQUEST,
            type='error'
        )
        raise ValueError('Unable to delete a protected file.')



from ftw.upgrade import UpgradeStep


class UpdateRolemapWithNewPermission(UpgradeStep):
    """Update rolemap with new permission (ftw.file: Edit date fields).
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.file.interfaces import IFileDownloadedEvent
from ftw.file.testing import FTW_FILE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from ftw.testbrowser import LIB_TRAVERSAL
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from StringIO import StringIO
from unittest import TestCase
from webdav.common import rfc1123_date
from zope.component import eventtesting
from zope.component import queryMultiAdapter

import AccessControl
import transaction


class TestFileDownload(TestCase):

    layer = FTW_FILE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.file = StringIO(1234 * 'dummy')
        setattr(self.file, 'filename', 'file.doc')
        self.portal.invokeFactory('File', 'f1', file=self.file)
        self.context = self.portal.f1
        # We need a modification date in ._p_mtime
        transaction.commit()

    def tearDown(self):
        super(TestFileDownload, self).tearDown()
        eventtesting.clearEvents()

    def index_html(self, disposition='attachment'):
        request = self.layer['request']
        response = request.RESPONSE
        return self.context.getField('file').index_html(self.context,
            REQUEST=request, RESPONSE=response, disposition=disposition)

    def test_content_length_header(self):
        self.index_html()
        response = self.layer['request'].RESPONSE
        self.assertEqual(
            '6170',
            response.getHeader('Content-Length')
        )

    def test_handle_if_modified_since_requests(self):
        self.layer['request'].environ.update({
            'HTTP_IF_MODIFIED_SINCE': rfc1123_date(self.context._p_mtime+1)
        })
        self.index_html()
        response = self.layer['request'].RESPONSE
        self.assertEqual(304, response.getStatus())

    def test_handle_range_requests(self):
        self.layer['request'].environ.update({'HTTP_RANGE': 'bytes=0-'})
        self.index_html()
        response = self.layer['request'].RESPONSE
        self.assertEqual(206, response.getStatus())
        self.assertEqual(
            'bytes 0-6169/6170',
            response.getHeader('Content-Range'))

    def test_download_returns_stream_iterator(self):
        self.assertEqual(self.file.getvalue(),self.index_html().next())

    def test_file_download_notification(self):
        self.index_html()
        events = [e for e in eventtesting.getEvents()
                  if IFileDownloadedEvent.providedBy(e)]
        self.assertEqual(1, len(events))

    def test_file_download_no_notification_when_system(self):
        _original_security = AccessControl.getSecurityManager()

        _system_user = AccessControl.SecurityManagement.SpecialUsers.system
        AccessControl.SecurityManagement.newSecurityManager(None, _system_user)
        self.index_html()
        events = [e for e in eventtesting.getEvents()
                  if IFileDownloadedEvent.providedBy(e)]
        self.assertEqual(0, len(events))
        AccessControl.SecurityManagement.setSecurityManager(
            _original_security)
        _original_security = None

    @browsing
    def test_inline_download(self, browser):
        browser.login().visit(self.context, view='@@download')
        self.assertEquals('attachment; filename="file.doc"; filename=file.doc*=UTF-8',
                          browser.headers['content-disposition'])

        browser.visit(self.context, view='@@download?inline=true')
        self.assertEquals('inline; filename="file.doc"; filename=file.doc*=UTF-8',
                          browser.headers['content-disposition'])

    @browsing
    def test_head_request(self, browser):
        browser.default_driver = LIB_TRAVERSAL

        browser.login()

        # Accessing "@@download" on the file would normally result in a 302,
        # telling us to go to "@@download/file/file.doc" instead.
        # But since the testbrowser follows the redirects, we get a 200 instead.
        browser.open(self.context, view='@@download', method='HEAD')
        self.assertEqual(200, browser.status_code)
        self.assertEqual(
            'http://nohost/plone/f1/@@download/file/file.doc',
            browser.url
        )

        # Open the correct url used to access the file.
        browser.open(self.context, view='@@download/file/file.doc', method='HEAD')
        self.assertEqual('200 Ok', browser.headers['status'])
        self.assertEqual('6170', browser.headers['content-length'])
        self.assertEqual('attachment; filename="file.doc"; filename=file.doc*=UTF-8',
                         browser.headers['content-disposition'])
        self.assertEqual('bytes', browser.headers['accept-ranges'])
        self.assertEqual('application/msword', browser.headers['content-type'])


class TestFileDownloadView(TestCase):

    layer = FTW_FILE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.file = StringIO(1234 * 'dummy')
        setattr(self.file, 'filename', 'file.doc')
        self.portal.invokeFactory('File', 'f1', file=self.file)
        self.context = self.portal.f1

        # Patch our index_html method for simpler testing
        def index_html(self, *args, **kwargs):
            return 'My index_html'
        field = self.context.getField('file')
        self.index_html_orig = field.index_html
        field.index_html = index_html.__get__(field, field.__class__)

    def tearDown(self):
        # Revert patch
        field = self.context.getField('file')
        field.index_html = self.index_html_orig

    def test_download_view_uses_our_index_html(self):
        view = queryMultiAdapter((self.context, self.layer['request']),
                                 name=u'download')
        view.fieldname = 'file'
        self.assertEqual('My index_html', view())



from Products.TinyMCE.adapters.JSONDetails import JSONDetails
import json


class FtwJSONDetails(JSONDetails):

    def getDetails(self):
        details_json = super(FtwJSONDetails, self).getDetails()
        details = json.loads(details_json)
        # patch default scale on ftw.file
        for scale in details.get('scales', []):

            if scale['title'] == 'Original':
                scale['value'] = '/@@images/image'
                break

        details_json = json.dumps(details)
        return details_json



try:
    from Products.contentmigration.archetypes import InplaceATItemMigrator
    from Products.contentmigration.migrator import BaseInlineMigrator
    from Products.contentmigration.walker import CustomQueryWalker
    haveContentMigrations = True
    BaseMigrator = InplaceATItemMigrator
    InlineMigrator = BaseInlineMigrator
except ImportError:
    BaseMigrator = object
    InlineMigrator = object
    haveContentMigrations = False

from plone.app.blob.migrations import migrate
from Products.CMFCore.utils import getToolByName
from Products.contentmigration.catalogpatch import applyCatalogPatch
from Products.contentmigration.catalogpatch import removeCatalogPatch
from Products.contentmigration.common import HAS_LINGUA_PLONE
from Products.contentmigration.archetypes import BaseATMigrator
from zope.app.component.hooks import getSite
from zope.component import getUtility
from Acquisition import aq_inner, aq_base, aq_parent
from zope.component.interfaces import IFactory
from Products.Archetypes.interfaces import IReferenceable
from Products.Archetypes.ArchetypeTool import getType
from Products.Archetypes.config import REFERENCE_ANNOTATION


class FtwFileMigrator(BaseMigrator):
    """ Migration of file content to blob content type
    """
    src_portal_type = 'ftw_File'
    src_meta_type = 'ftw_File'
    dst_portal_type = 'File'
    dst_meta_type = 'FtwFile'

    # migrate all fields except 'file', which needs special handling...
    fields_map = {
        'file': None,
    }

    def migrate_data(self):
        oldfield = self.old.getField('file')
        value = oldfield.get(self.old)
        self.new.getField('file').getMutator(self.new)(value)

    def last_migrate_reindex(self):
        self.new.reindexObject(idxs=['object_provides', 'portal_type',
                                     'Type', 'UID'])
    # XXX: doesn't work yet, trying transmogrifier instead
    # def last_migrate_versions(self):
    #     atool = getToolByName(self.old, 'portal_archivist')
    #     history = atool.getHistory(self.old)
    #     for version in history:
    #         obj = version.data.object
    #         migrator = FtwFileVersionMigrator(obj)
    #         migrator.migrate()


class FtwFileVersionMigrator(BaseATMigrator):
    src_portal_type = 'ftw_File'
    src_meta_type = 'ftw_File'
    dst_portal_type = 'File'
    dst_meta_type = 'FtwFile'

    # migrate all fields except 'file', which needs special handling...
    fields_map = {
        'file': None,
    }

    def __init__(self, obj, src_portal_type=None, dst_portal_type=None,
                 **kwargs):
        self.old = aq_inner(obj)
        self.orig_id = self.old.getId()
        self.old_id = '%s_MIGRATION_' % self.orig_id
        self.new = None
        self.new_id = self.orig_id
        self.parent = aq_parent(self.old)
        if src_portal_type is not None:
            self.src_portal_type = src_portal_type
        if dst_portal_type is not None:
            self.dst_portal_type = dst_portal_type
        self.kwargs = kwargs

        # safe id generation
        while hasattr(aq_base(self.parent), self.old_id):
            self.old_id += 'X'

    def renameOld(self):
        """Renames the old object
        """
        setattr(self.parent, self.old_id, self.old)
        delattr(self.parent, self.orig_id)

    def createNew(self):
        """Create the new object
        """
        portal = getSite()
        typesTool = getToolByName(portal, 'portal_types')
        fti = typesTool.getTypeInfo(self.dst_portal_type)
        factory = getUtility(IFactory, fti.factory)
        obj = factory(self.new_id, **self.schema)
        if hasattr(obj, '_setPortalTypeName'):
            obj._setPortalTypeName(self.getId())
        setattr(self.parent, self.new_id, obj)
        self.new = getattr(aq_inner(self.parent).aq_explicit, self.new_id)

    def remove(self):
        """Removes the old item
        """
        delattr(self.parent, self.old_id)

    def reorder(self):
        """Reorder the new object in its parent"""
        return

    def beforeChange_schema(self):
        """Load the values of fields from according to fields_map if present.
        Each key in fields_map is a field in the old schema and each
        value is a field in the new schema.  If fields_map isn't a
        mapping, each filed in the old schema will be migrated into
        the new schema.  Obeys field modes for readable and writable
        fields.  These values are then passed in as field kwargs into
        the constructor in the createNew method."""

        old_schema = self.old.Schema()

        portal = getSite()
        typesTool = getToolByName(portal, 'portal_types')
        fti = typesTool.getTypeInfo(self.dst_portal_type)
        archetype = getType(self.dst_meta_type, fti.product)
        new_schema = archetype['klass'].schema

        if self.only_fields_map:
            old_field_names = self.fields_map.keys()
        else:
            old_field_names = old_schema.keys()

        # Let the migrator handle the id and dates
        for omit_field_name in ['id', 'creation_date',
                                'modification_date']:
            if omit_field_name in old_field_names:
                old_field_names.remove(omit_field_name)

        kwargs = getattr(self, 'schema', {})
        for old_field_name in old_field_names:
            old_field = self.old.getField(old_field_name)
            new_field_name = self.fields_map.get(old_field_name,
                                                 old_field_name)

            if new_field_name is None:
                continue

            new_field = new_schema.get(new_field_name, None)
            if new_field is None:
                continue

            if ('r' in old_field.mode and 'w' in new_field.mode):
                accessor = (
                    getattr(old_field, self.accessor_getter)(self.old)
                    or old_field.getAccessor(self.old))
                value = accessor()
                kwargs[new_field_name] = value
        self.schema = kwargs

    def migrate_owner(self):
        """Migrates the zope owner
        """
        self.new._owner = self.old.getOwner(info=1)

    def migrate_localroles(self):
        """Migrate local roles
        """
        return

    def migrate_user_roles(self):
        """Migrate roles added by users
        """
        return

    def migrate_at_uuid(self):
        """Migrate AT universal uid
        """
        if not IReferenceable.providedBy(self.old):
            return  # old object doesn't support AT uuids

        uid = self.old.UID()
        self.new._setUID(uid)

    def migrate_references(self):
        """Migrate references annotation."""
        # Restor the references annotation
        if hasattr(self, REFERENCE_ANNOTATION):
            at_references = getattr(self, REFERENCE_ANNOTATION)
            setattr(self.new, REFERENCE_ANNOTATION, at_references)


class FtwFileMigrationWalker(CustomQueryWalker):
    """
    """

    def walk(self):
        """Walks around and returns all objects which needs migration

        :return: objects (with acquisition wrapper) that needs migration
        :rtype: generator
        """
        catalog = self.catalog
        query = self.additionalQuery.copy()
        query['portal_type'] = self.src_portal_type
        query['meta_type'] = self.src_meta_type

        if HAS_LINGUA_PLONE and 'Language' in catalog.indexes():
            query['Language'] = 'all'

        for brain in catalog(query)[:1000]:
            obj = brain.getObject()

            if self.callBefore is not None and callable(self.callBefore):
                if self.callBefore(obj, **self.kwargs) is False:
                    continue

            try:
                state = obj._p_changed
            except:
                state = 0
            if obj is not None:
                yield obj
                # safe my butt
                if state is None:
                    obj._p_deactivate()


def getFtwFileMigrationWalker(context):
    """ set up migration walker """
    portal = getToolByName(context, 'portal_url').getPortalObject()
    return FtwFileMigrationWalker(
        portal, FtwFileMigrator, transaction_size=100)


def migrateFtwFiles(context):

    portal = getToolByName(context, 'portal_url').getPortalObject()
    out = ''
    try:
        catalog_class = applyCatalogPatch(portal)
        out = migrate(context, walker=getFtwFileMigrationWalker)
    finally:
        removeCatalogPatch(catalog_class)

    return out


# version migration using collective.jsonify
import urllib
import urlparse
import logging
import json
import base64
logger = logging.getLogger('ftw.file.migration')


class UrllibrpcException(Exception):
    """Raised when reading an url fails.
    """

    def __init__(self, code, url):
        self.code = code
        self.url = url

    def __str__(self):
        return '%s:%s' % (self.code, self.url)


class Urllibrpc(object):

    def __init__(self, url, username, password):
        self.url = url
        self.username = username
        self.password = password

    def __getattr__(self, item):

        def callable():
            scheme, netloc, path, params, query, fragment = urlparse.urlparse(
                self.url)
            if '@' not in netloc:
                netloc = '%s:%s@%s' % (self.username, self.password, netloc)
            if path.endswith("/"):
                path = path[:-1]
            path = path + '/' + item
            url = urlparse.urlunparse(
                (scheme, netloc, path, params, query, fragment))
            f = urllib.urlopen(url)
            content = f.read()
            if f.getcode() != 200:
                raise UrllibrpcException(f.getcode(), f.geturl())
            f.close()
            return content
        return callable


def migrate_file_versions(
    context, remote_user='admin', remote_password='admin'):
    """Migrate file version using collective.jsonify as data source.
    """
    remote_url = 'http://127.0.0.1:9080'
    catalog = getToolByName(context, 'portal_catalog')
    repo_tool = getToolByName(context, 'portal_repository')
    files = catalog(portal_type='File')
    for file_ in files:
        url = '%s%s' % (remote_url, file_.getPath())
        remote = Urllibrpc(url, remote_user, remote_password)

        try:
            remote_data = remote.get_item_with_file()
        except UrllibrpcException, e:
            logger.warn("Failed reading url '%s' with error code %s." %
                         (e.url, e.code))
            continue

        try:
            remote_data = json.loads(remote_data)
        except ValueError:
            logger.warn(
                "No JSON object could be decoded using data from url '%s'." %
                url)
            continue

        obj = file_.getObject()
        field = obj.getField('file')
        file_data = to_utf8(remote_data.get('_datafield_file'))

        # a list indicates multiple versions
        if isinstance(file_data, list):
            principals = []
            for version in file_data:
                field.set(obj, base64.b64decode(version.get('data')))
                field.getUnwrapped(obj).filename = version.get('filename')
                field.getUnwrapped(obj).content_type = version.get(
                    'content_type')
                if version.get('version_id') != 'WORKING_COPY':
                    principals.append(
                        version['version_sysmetadata']['principal'])
                    repo_tool._recursiveSave(obj, {},
                                         version.get('version_sysmetadata'),
                                         autoapply=repo_tool.autoapply)
                else:
                    mdate_mutator = obj.getField(
                        'modification_date').getMutator(obj)
                    mdate_mutator(remote_data.get('modification_date'))

            # fix principal in metadata
            hm = repo_tool.getHistoryMetadata(obj)
            for version_id, metadata in hm._full.items():
                if version_id < len(principals):
                    metadata['metadata']['sys_metadata']['principal'] = \
                        principals[version_id]
                    hm._full[version_id] = metadata

        # Do nothing if we didn't get multiple versions


def to_utf8(obj):
    """Recursively convert unicode to uft-8 strings in lists and dicts."""
    if isinstance(obj, unicode):
        obj = obj.encode('utf8')
    elif isinstance(obj, list):
        for i, v in enumerate(obj):
            obj[i] = to_utf8(v)
    elif isinstance(obj, dict):
        new_obj = {}
        for k, v in obj.iteritems():
            if isinstance(k, unicode):
                k = k.encode('utf8')
            new_obj[k] = to_utf8(v)
        obj = new_obj
    return obj



import transaction

from ftw.file.testing import FTW_FILE_FUNCTIONAL_TESTING
from plone.app.testing import TEST_USER_ID
from Products.CMFCore.utils import getToolByName
from plone.app.testing import setRoles
from unittest import TestCase
from ftw.builder import create
from ftw.builder import Builder
from ftw.testbrowser import browsing


class TestFileName(TestCase):

    layer = FTW_FILE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.context = create(Builder('file').attach_asset("transparent.gif"))
        self._set_allowAnonymousViewAbout_property(True)

    def _set_allowAnonymousViewAbout_property(self, value):
        site_props = getToolByName(self.context,
                                   'portal_properties').site_properties
        site_props._updateProperty('allowAnonymousViewAbout', value)
        transaction.commit()

    def is_author_visible(self, page):
        return 'Author' in page.css(".fileListing th").text

    @browsing
    def test_logged_in_user_sees_author_when_allowAnonymousViewAbout(self,
                                                                     browser):
        page = browser.login().visit(self.context, view="file_view")
        self.assertTrue(self.is_author_visible(page),
                        'Logged in user should see author if '
                        'allowAnonymousViewAbout is True.')

    @browsing
    def test_logged_in_user_sees_author_when_not_allowAnonymousViewAbout(self, browser):
        self._set_allowAnonymousViewAbout_property(False)
        page = browser.login().visit(self.context, view="file_view")
        self.assertTrue(self.is_author_visible(page),
                        'Logged in user should see author if '
                        'allowAnonymousViewAbout is False.')

    @browsing
    def test_anonymous_user_sees_author_when_allowAnonymousViewAbout(self,
                                                                     browser):
        page = browser.visit(self.context, view="file_view")
        self.assertTrue(self.is_author_visible(page),
                        'Anonymous user should see author if '
                        'allowAnonymousViewAbout is True.')

    @browsing
    def test_anonymous_user_dont_sees_author_when_not_allowAnonymousViewAbout(self, browser):
        self._set_allowAnonymousViewAbout_property(False)
        page = browser.visit(self.context, view="file_view")
        self.assertFalse(self.is_author_visible(page),
                         'Anonymous user should not see author if '
                         'allowAnonymousViewAbout is False.')



"""Main product initializer
"""
from Products.Archetypes import atapi
from Products.CMFCore.utils import ContentInit
from ftw.file import config
from zope.i18nmessageid import MessageFactory

fileMessageFactory = MessageFactory('ftw.file')


def initialize(context):
    """Initializer called when used as a Zope 2 product."""

    # Retrieve the content types that have been registered with Archetypes.
    content_types, constructors, ftis = atapi.process_types(
        atapi.listTypes(config.PROJECTNAME),
        config.PROJECTNAME)

    # Now initialize all these content types.
    for atype, constructor in zip(content_types, constructors):
        ContentInit('%s: %s' % (config.PROJECTNAME, atype.portal_type),
                    content_types=(atype, ),
                    permission=config.ADD_PERMISSIONS[atype.portal_type],
                    extra_constructors=(constructor, ),
                    ).initialize(context)



from Products.Archetypes.utils import shasattr
from ftw.file.interfaces import IFtwFileField
from plone.app.blob.mixins import ImageFieldMixin
from plone.app.imaging.traverse import DefaultImageScaleHandler
from zope.component import adapts
from zope.interface import implementsOnly


class FtwFileImageScaleHandler(DefaultImageScaleHandler):
    adapts(IFtwFileField)

    def getScale(self, instance, scale):
        if self.context.get(instance).content_type.startswith('image/'):
            return super(FtwFileImageScaleHandler, self).getScale(instance,
                                                                  scale)
        else:
            return None


class ImagingMixin(ImageFieldMixin):
    """The goal of this mixin is to get the funtionality and benefits of an
    ImageField while not restricting the current funtionality of the field.
    The problem which surfaced was, that we weren't able to upload files
    anymore that weren't images, because a validator was registered for the
    ImageField to ensure only images are uploaded.
    Therefore we inherit from the ImageFieldMixin and remove its implemented
    interfaces since they are often used to restrict certain behaviours.
    """

    implementsOnly()

    def getScale(self, instance, scale=None, **kwargs):
        """Use Original Archetypes implementation since the one of the mixin
        leads to a loop.
        """

        if scale is None:
            return self.get(instance, **kwargs)
        else:
            id = self.getName() + '_' + scale
            try:
                image = self.getStorage(instance).get(id, instance, **kwargs)
            except AttributeError:
                return ''
            image = self._wrapValue(instance, image, **kwargs)
            if shasattr(image, '__of__', acquire=True) and not kwargs.get(
                    'unwrapped', False):
                return image.__of__(instance)
            else:
                return image



from ftw.builder import Builder
from ftw.builder import create
from ftw.file.testing import FTW_FILE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_PASSWORD
from Products.CMFCore.utils import getToolByName
from plone.registry.interfaces import IRegistry
from unittest import TestCase
from zope.component import getUtility


class TestDownloadRedirection(TestCase):

    layer = FTW_FILE_FUNCTIONAL_TESTING

    def setUp(self):
        portal = self.layer['portal']
        acl_users = getToolByName(portal, 'acl_users')
        acl_users.userFolderAddUser('reader', TEST_USER_PASSWORD,
                                    ['Reader'], [])
        acl_users.userFolderAddUser('contributor', TEST_USER_PASSWORD,
                                    ['Contributor'], [])

        login(portal, 'contributor')

    @browsing
    def test_redirects_to_details_view_when_i_can_edit(self, browser):
        obj = create(Builder('file'))
        browser.login('contributor').visit(obj)

        self.assertEquals('http://nohost/plone/file/view',
                          browser.url)

    @browsing
    def test_redirects_to_download_when_i_cannot_edit(self, browser):
        obj = create(Builder('file')
                     .with_dummy_content())
        browser.login('reader').visit(obj)

        self.assertEquals(
            'http://nohost/plone/file/@@download/file/test.doc',
            browser.url)

    @browsing
    def test_redirects_to_details_when_redirect_to_download_is_disabled(self, browser):
        registry = getUtility(IRegistry)
        registry['ftw.file.disable_download_redirect'] = True

        obj = create(Builder('file'))

        # Redirect users having the permission to edit file.
        browser.login('contributor').visit(obj)

        self.assertEquals('http://nohost/plone/file/view',
                          browser.url)

        # Redirect users not having the permission to edit file.
        browser.login('reader').visit(obj)

        self.assertEquals('http://nohost/plone/file/view',
                          browser.url)


class TestDownloadRedirectToURLWithFilename(TestCase):

    layer = FTW_FILE_FUNCTIONAL_TESTING

    def setUp(self):
        portal = self.layer['portal']
        setRoles(portal, TEST_USER_ID, ['Manager'])

    @browsing
    def test_download_view_redirects_to_url_with_filename(self, browser):
        obj = create(Builder('file')
                     .titled('Document')
                     .attach_file_containing('PDF CONTENT', name='doc.pdf'))

        browser.login().visit(obj, view='@@download')

        self.assertEquals(
            'http://nohost/plone/document/@@download/file/doc.pdf',
            browser.url)

        self.assertEquals('PDF CONTENT', browser.contents)

    @browsing
    def test_download_method_redirects_to_url_with_filename(self, browser):
        obj = create(Builder('file')
                     .titled('Document')
                     .attach_file_containing('PDF CONTENT', name='doc.pdf'))

        browser.login().visit(obj, view='download')

        self.assertEquals(
            'http://nohost/plone/document/@@download/file/doc.pdf',
            browser.url)

        self.assertEquals('PDF CONTENT', browser.contents)

    @browsing
    def test_supports_special_characters_and_umlauts_in_filename(self, browser):
        obj = create(
            Builder('file')
            .titled('Document')
            .attach_file_containing(
                'PDF CONTENT',
                name='w\xc3\xb6rter & bilder.pdf'))

        browser.login().visit(obj, view='download')

        self.assertEquals(
            'http://nohost/plone/document/@@download/'
            'file/w%C3%B6rter%20%26%20bilder.pdf',
            browser.url)

        self.assertEquals('PDF CONTENT', browser.contents)

    @browsing
    def test_supports_unicode_special_characters_in_filename(self, browser):
        obj = create(
            Builder('file')
            .titled('Document')
            .attach_file_containing(
                'PDF CONTENT',
                name='w\xc3\xb6rter & bilder.pdf'.decode('utf-8')))

        browser.login().visit(obj, view='download')

        self.assertEquals(
            'http://nohost/plone/document/@@download/'
            'file/w%C3%B6rter%20%26%20bilder.pdf',
            browser.url)

        self.assertEquals('PDF CONTENT', browser.contents)

    @browsing
    def test_remove_percent_signs_form_filename(self, browser):
        obj = create(
            Builder('file')
            .titled('Document')
            .attach_file_containing(
                'PDF CONTENT',
                name='50%to80%.pdf'.decode('utf-8')))

        browser.login().visit(obj, view='download')

        self.assertEquals(
            'http://nohost/plone/document/@@download/'
            'file/50to80.pdf',
            browser.url)

        self.assertEquals('PDF CONTENT', browser.contents)



from plone.app.blob.interfaces import IBlobField
from zope.component.interfaces import IObjectEvent
from zope.interface import Attribute
from zope.interface import Interface


class IFile(Interface):
    """File marker interface.
    """


class IFileDownloadedEvent(IObjectEvent):
    """An event fired when a file is downloaded"""

    context = Attribute("The file object that was downloaded")


class IFtwFileField(IBlobField):
    """FileField marker"""



from ftw.builder import builder_registry
from ftw.builder.content import ATFileBuilder
from path import Path


class CustomFileBuilder(ATFileBuilder):

    def attach_asset(self, filename):
        return self.attach_file_containing(
            self._asset(filename).bytes(), filename)

    def _asset(self, filename):
        path = Path(__file__).dirname().realpath().joinpath('assets', filename)
        return path


builder_registry.register('file', CustomFileBuilder, force=True)



from ftw.upgrade import UpgradeStep


class AddRegistryRecord(UpgradeStep):
    """Add registry record (disable download redirect).
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.builder import Builder
from ftw.builder import create
from ftw.file.testing import FTW_FILE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import statusmessages
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from StringIO import StringIO
from unittest import TestCase
from ZPublisher.HTTPRequest import HTTPRequest
from ZPublisher.HTTPResponse import HTTPResponse


class TestFileName(TestCase):

    layer = FTW_FILE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('File', 'f1')
        self.context = self.portal.f1

    def get_response_for(self, filename='foo.pdf', response=HTTPResponse(),
                         request=None):
        if not request:
            request = HTTPRequest('', dict(HTTP_HOST='nohost:8080'),
                                  {}, response)
        self.set_filedata(filename)
        self.context.getField('file').index_html(self.context, REQUEST=request,
                                                 RESPONSE=response,
                                                 disposition='attachment')
        return response

    def set_filedata(self, filename):
        data = StringIO(100 * "dummy")
        setattr(data, 'filename', filename)
        self.context.setFile(data)

    def test_whitespace(self):
        response = self.get_response_for(filename='ein file.doc')
        self.assertEqual('attachment; filename="ein file.doc"; filename=ein file.doc*=UTF-8',
                         response.getHeader('Content-disposition'))

    def test_umlauts(self):
        response = self.get_response_for(
            filename='Gef\xc3\xa4hrliche Zeichen.doc')
        self.assertEqual(
            'attachment; filename="Gef\xc3\xa4hrliche Zeichen.doc"; filename=Gef\xc3\xa4hrliche Zeichen.doc*=UTF-8',
            response.getHeader('Content-disposition'))

    def test_unicode(self):
        response = self.get_response_for(filename=u'\xfcber.html')
        self.assertEqual(
            'attachment; filename="\xc3\xbcber.html"; filename=\xc3\xbcber.html*=UTF-8',
            response.getHeader('Content-disposition'))

    def test_msie(self):
        request = HTTPRequest('', dict(HTTP_HOST='nohost:8080',
                                       HTTP_USER_AGENT='MSIE'), {})
        response = self.get_response_for(filename=u'\xfcber.html',
                                         request=request)

        self.assertEqual('attachment; filename=\xc3\xbcber.html; filename*=UTF-8\'\'%C3%BCber.html',
                         response.getHeader('Content-disposition'))
        response = self.get_response_for(
            filename=u'\xfcber\xe2\x80\x93uns.html', request=request)
        self.assertEqual(
            'attachment; filename=\xc3\xbcber\xc3\xa2\xc2\x80\xc2\x93uns.html; filename*=UTF-8\'\'%C3%BCber%C3%A2%C2%80%C2%93uns.html',
            response.getHeader('Content-disposition'))

    def test_get_origin_filename_has_no_extension(self):
        self.set_filedata('dummyfile.txt')
        self.assertEqual('dummyfile', self.context.getOriginFilename())

    def test_set_origin_filename_overrides_the_filename_and_keeps_the_extension(self):
        self.set_filedata('dummyfile.txt')
        self.context.setOriginFilename(u'\xfcber')
        self.assertEqual('\xc3\xbcber.txt', self.context.getFilename())

    def test_set_origin_filename_does_not_override_filename_if_its_empty(self):
        self.set_filedata('dummyfile.txt')
        self.context.setOriginFilename('')
        self.assertEqual('dummyfile.txt', self.context.getFilename())

    def test_set_origin_filename_if_no_filename_exists(self):
        self.assertEqual(None, self.context.getFilename())
        self.context.setOriginFilename('testfile')
        self.assertEqual(None, self.context.getFilename())

    def test_get_origin_filename_if_no_filename_exists(self):
        self.assertEqual(None, self.context.getOriginFilename())

    def test_get_origin_filename_if_no_extension_exists(self):
        self.set_filedata('dummyfile')
        self.assertEqual('dummyfile', self.context.getOriginFilename())

    def test_set_origin_filename_if_no_extension_exists(self):
        self.set_filedata('dummyfile')
        self.context.setOriginFilename(u'\xfcber')
        self.assertEqual('\xc3\xbcber', self.context.getFilename())

    @browsing
    def test_origin_filename_is_set_if_no_file_is_uploaded(self, browser):
        existingfile = create(Builder('file').with_dummy_content()
                              .titled('The File'))
        browser.login().open(existingfile, view='edit')
        browser.fill({'Filename': 'newFilename'}).submit()
        self.assertEquals('newFilename.doc', existingfile.getFilename())

    @browsing
    def test_origin_filename_is_filename_of_uploaded_file(self, browser):
        existingfile = create(Builder('file').with_dummy_content())
        browser.login().open(existingfile, view='edit')
        browser.fill({
            'file_delete': '',
            'file_file': ('Raw file data', 'newfile.txt', 'text/plain')
        }).submit()
        self.assertEqual('newfile.txt', existingfile.getFilename())

    @browsing
    def test_origin_filename_cannot_contain_slash(self, browser):
        existingfile = create(Builder('file').with_dummy_content()
                              .titled('The File'))

        browser.login()

        # Set an origin filename without slash. This must work.
        browser.open(existingfile, view='edit')
        browser.fill({'Filename': 'Foobar'}).submit()
        statusmessages.assert_no_error_messages()
        self.assertEqual('Foobar.doc', existingfile.getFilename())

        # Set an origin filename containing a slash. This must fail.
        browser.open(existingfile, view='edit')
        browser.fill({'Filename': 'Foo / bar'}).submit()
        statusmessages.assert_message('Please correct the indicated errors.')
        self.assertEqual('Foobar.doc', existingfile.getFilename())



from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.file.utils import FileMetadata
from Products.Five import BrowserView


class FileView(BrowserView):
    """ View for ftw.file """

    template = ViewPageTemplateFile('file_view.pt')

    def __call__(self):
        return self.template()

    def get_image_tag(self):
        return self.metadata.get_image_tag(
            fieldname='file', width=200)

    @property
    def metadata(self):
        if not hasattr(self, '_metadata'):
            self._metadata = FileMetadata(self.context)
        return self._metadata



from DateTime import DateTime
from ftw.builder import Builder
from ftw.builder import create
from ftw.file.testing import FTW_FILE_FUNCTIONAL_TESTING
from ftw.file.testing import FTW_FILE_INTEGRATION_TESTING
from ftw.file.utils import FileMetadata
from ftw.file.utils import redirect_to_download_by_default
from plone import api
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from Products.CMFCore.utils import getToolByName
from unittest import TestCase


class TestRedirectToDownloadByDefault(TestCase):

    layer = FTW_FILE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Contributor'])

    def test_should_return_True_for_anonymous_users(self):
        login(self.portal, TEST_USER_NAME)
        obj = create(Builder('file'))
        logout()

        self.assertTrue(
            redirect_to_download_by_default(obj),
            'An anonymous user should be redirected to the downloads view.')

    def test_should_return_True_for_anonymous_users_with_forced_border(self):
        login(self.portal, TEST_USER_NAME)
        obj = create(Builder('file'))
        logout()

        obj.REQUEST.other['enable_border'] = True
        self.assertTrue(
            redirect_to_download_by_default(obj),
            'An anonymous user should be redirected to the downloads view, '
            'even when enable_border is True.')

        self.assertTrue(
            obj.REQUEST.get('enable_border'),
            'enable_border should not be removed from the request')

    def test_should_return_False_for_editors(self):
        login(self.portal, TEST_USER_NAME)
        obj = create(Builder('file'))
        setRoles(self.portal, TEST_USER_ID, ['Editor'])

        self.assertFalse(
            redirect_to_download_by_default(obj),
            'An editor should not be redirected to the downloads view.')

    def test_should_return_False_for_editors_in_previewish_mode(self):
        login(self.portal, TEST_USER_NAME)
        obj = create(Builder('file'))
        setRoles(self.portal, TEST_USER_ID, ['Editor'])

        obj.REQUEST.other['disable_border'] = True
        self.assertFalse(
            redirect_to_download_by_default(obj),
            'An editor should not be redirected to the downloads view, '
            'even when disable_border is True.')

        self.assertTrue(
            obj.REQUEST.get('disable_border'),
            'disable_border should not be removed from the request')


class FileMetadataBase(TestCase):

    layer = FTW_FILE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.dummyfile = create(Builder('file').with_dummy_content())


class TestFileMetadataGetImageTag(FileMetadataBase):

    def test_return_none_if_no_image_exists(self):
        self.assertEqual(
            None,
            FileMetadata(self.dummyfile).get_image_tag(
                fieldname="file", width=1, direction="down"))

    def test_return_scale_if_available(self):
        img_file = create(Builder('file').attach_asset('transparent.gif'))
        self.assertTrue(
            FileMetadata(img_file).get_image_tag(
                fieldname="file", width=1, direction="down"),
            "Should return the scale for this image")

    def test_return_none_if_scale_is_not_available(self):
        img_file = create(Builder('file').attach_asset('CCITT_1.TIF'))
        self.assertEqual(
            None,
            FileMetadata(img_file).get_image_tag("file", 1, 'down'))


class TestFileMetadataCanEdit(FileMetadataBase):

    def test_can_edit_if_has_permission(self):
        login(self.portal, TEST_USER_NAME)
        self.assertTrue(FileMetadata(self.dummyfile).can_edit)

    def test_cannot_edit_if_has_no_permission(self):
        logout()
        self.assertFalse(FileMetadata(self.dummyfile).can_edit)


class TestFileMetadataDocumentDate(FileMetadataBase):

    def test_return_document_date_if_exists(self):
        self.dummyfile.setDocumentDate(DateTime('10.05.2015 10:25'))
        self.assertEqual(
            u'Oct 05, 2015', FileMetadata(self.dummyfile).document_date)


class TestFileMetadataModifiedDate(FileMetadataBase):

    def test_return_modified_date_if_exists(self):
        self.dummyfile.setModificationDate(DateTime('10.05.2015 10:25'))
        self.assertEqual(
            u'Oct 05, 2015 10:25 AM',
            FileMetadata(self.dummyfile).modified_date)


class TestFileMetadataAuthor(FileMetadataBase):

    def test_return_fullname_and_url_if_exist_user_exist(self):
        api.user.create(
            email='chuck@norris.org',
            username='chuck',
            properties={'fullname': 'Chuck Norris'})

        self.dummyfile.setCreators(tuple(['chuck', ]))

        self.assertEqual(
            {'id': 'chuck',
             'name': 'Chuck Norris',
             'url': 'http://nohost/plone/author/chuck'},
            FileMetadata(self.dummyfile).author)

    def test_return_userid_and_url_if_user_exists_without_fullname(self):
        api.user.create(
            email='chuck@norris.org',
            username='chuck',
            properties={'fullname': ''})

        self.dummyfile.setCreators(tuple(['chuck', ]))

        self.assertEqual(
            {'id': 'chuck',
             'name': 'chuck',
             'url': 'http://nohost/plone/author/chuck'},
            FileMetadata(self.dummyfile).author)

    def test_return_userid_and_no_url_if_user_does_not_exist(self):
        self.dummyfile.setCreators(tuple(['chuck', ]))

        self.assertEqual(
            {'id': 'chuck',
             'name': 'chuck',
             'url': ''},
            FileMetadata(self.dummyfile).author)


class TestFileMetadataShowAuthor(FileMetadataBase):

    def test_true_if_logged_in_and_allow_AnonymousViewAbout_is_true(self):
        self._login(True)
        self._set_allowAnonymousViewAbout(True)

        self.assertTrue(
            FileMetadata(self.dummyfile).show_author,
            'Logged in user should see author if allowAnonymousViewAbout '
            'is True.')

    def test_true_if_anonymous_and_allow_AnonymousViewAbout_is_true(self):
        self._login(False)
        self._set_allowAnonymousViewAbout(True)

        self.assertTrue(
            FileMetadata(self.dummyfile).show_author,
            'Anonymous user user should see author if allowAnonymousViewAbout '
            'is True.')

    def test_true_if_logged_in_and_allow_AnonymousViewAbout_is_false(self):
        self._login(True)
        self._set_allowAnonymousViewAbout(False)

        self.assertTrue(
            FileMetadata(self.dummyfile).show_author,
            'Logged in user should see author if allowAnonymousViewAbout '
            'is False.')

    def test_false_if_anonymous_and_allow_AnonymousViewAbout_is_false(self):
        self._login(False)
        self._set_allowAnonymousViewAbout(False)

        self.assertFalse(
            FileMetadata(self.dummyfile).show_author,
            'Anonymous user should not see author if allowAnonymousViewAbout '
            'is False.')

    def _set_allowAnonymousViewAbout(self, value):
        site_props = getToolByName(
            self.portal, 'portal_properties').site_properties
        site_props.allowAnonymousViewAbout = value

    def _login(self, value):
        if value:
            login(self.portal, TEST_USER_NAME)
        else:
            logout()



from AccessControl import ClassSecurityInfo
from DateTime import DateTime
from ftw.calendarwidget.browser.widgets import FtwCalendarWidget
from ftw.file import fileMessageFactory as _
from ftw.file.config import PROJECTNAME
from ftw.file.fields import FileField
from ftw.file.interfaces import IFile
from ftw.file.utils import is_image
from ftw.file.utils import redirect_to_download_by_default
from ftw.journal.interfaces import IWorkflowHistoryJournalizable
from logging import getLogger
from os import path
from plone.app.blob.field import BlobMarshaller
from Products.Archetypes import atapi
from Products.Archetypes.BaseContent import BaseContent
from Products.Archetypes.Widget import StringWidget
from Products.ATContentTypes.config import ICONMAP
from Products.ATContentTypes.content.file import ATFile
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.interfaces import IFileContent
from Products.CMFCore.permissions import ModifyPortalContent
from Products.CMFCore.permissions import View
from Products.CMFCore.utils import getToolByName
from Products.MimetypesRegistry.common import MimeTypeException
from Products.statusmessages.interfaces import IStatusMessage
from Products.validation import V_REQUIRED, UnknowValidatorError
from Products.validation.config import validation
from Products.validation.validators import RegexValidator
from urllib import quote
from ZODB.POSException import ConflictError
from zope.interface import implements

import logging
import transaction


origin_filename_validator = RegexValidator(
    'isSafeOriginFilename',
    r'^[^\/]*$',
    title='',
    description='',
    errmsg=_(
        u'origin_filename_validator_error',
        default=u'The filename must not contain "/".'
    )
)

validation.register(origin_filename_validator)


FileSchema = ATContentTypeSchema.copy() + atapi.Schema((
    FileField(
        'file',
        required=True,
        primary=True,
        searchable=True,
        languageIndependent=True,
        index_method='getIndexValue',
        storage=atapi.AnnotationStorage(migrate=True),
        validators=(('isNonEmptyFile', V_REQUIRED),
                    ('checkFileMaxSize', V_REQUIRED)),
        widget=atapi.FileWidget(
            description='',
            label=_(u'label_file', default=u'File'),
            show_content_type=False,
        ),
    ),
    atapi.StringField(
        name='originFilename',
        required=False,
        validators=('isSafeOriginFilename',),
        widget=StringWidget(
            helper_js=(
                "++resource++ftw.file.resources/hideOriginFilenameField.js", ),
            label=_(u'label_origin_filename', default=u'Filename'),
            description=_(
                u'help_origin_filename',
                default=u"Insert a filename if you want to change the "
                        "original filename. The extension (i.e. .docx) "
                        "will not be modified. Please do not enter \"/\".")
        ),
    ),
    atapi.DateTimeField(
        'documentDate',
        required=True,
        default_method=DateTime,
        widget=FtwCalendarWidget(
            label=_(u'label_document_date', default=u'Document Date'),
            description=_(u'help_document_date', default=u'')
        ),
    ),
    atapi.BooleanField(
        'isProtected',
        required=False,
        default=False,
        widget=atapi.BooleanWidget(
            label=_(u'label_is_protected', default=u'Protected'),
            description=_(u'help_is_protected',
                          default=u'The file cannot cannot be deleted if this option is checked.'),
        ),
        write_permission='ftw.file: Protect file',
    ),
))

# Register BlobMarshaller for the marshall layer so it gets
# used when de-marshalling files that are saved with the
# ExternalEditor / WebDav PUT
# This fixes https://extranet.4teamwork.ch/intranet/10-interne-projekte/
# 4teamwork-egov/tracker-4teamwork-egov/465
FileSchema.registerLayer('marshall', BlobMarshaller())

FileSchema['documentDate'].widget.show_hm = False

schematas = ['categorization', 'ownership', 'settings', 'creators']
for f in FileSchema.keys():
    field_ = FileSchema[f]
    if field_.schemata in schematas:
        field_.write_permission = 'ftw.file: Edit advanced fields'

for f in FileSchema.keys():
    field_ = FileSchema[f]
    if field_.schemata == 'dates':
        field_.write_permission = 'ftw.file: Edit date fields'


class File(ATFile):
    """A file content type based on blobs.
    """
    implements(IFile, IFileContent, IWorkflowHistoryJournalizable)

    meta_type = "FtwFile"
    schema = FileSchema
    security = ClassSecurityInfo()

    security.declareProtected(View, 'index_html')
    def index_html(self, REQUEST, RESPONSE):
        """ Redirect to the default view or download view """

        if redirect_to_download_by_default(self):
            return RESPONSE.redirect(self.absolute_url() + '/download')
        else:
            return RESPONSE.redirect(self.absolute_url() + "/view")

    security.declareProtected(View, 'download')
    def download(self, REQUEST=None, RESPONSE=None):
        """Download the file (use default index_html)
        """
        return self.restrictedTraverse('@@download')()

    security.declareProtected(ModifyPortalContent, 'setFile')
    def setFile(self, value, **kwargs):
        """
        Catch *any* file upload so we can ensure virus scanning has been
        done if it's enabled.
        """
        if '_initializing_' not in kwargs:
            try:
                result = validation('isVirusFree', value, REQUEST=self.REQUEST)
                # Note: the validator returns True for OK, and a (truish) string
                # for virus found :-/
                if result is not True:
                    transaction.doom()
                    # Duplicate messages should not be shown
                    # (see Products.statusmessages.adapter._encodeCookieValue)
                    IStatusMessage(self.REQUEST).add(result, 'error')
            except UnknowValidatorError:
                # collective.clamav is not installed
                pass

        super(File, self).setFile(value, **kwargs)
        # Note: standard archetypes validation doesn't happen until after this returns

    security.declarePrivate('getIndexValue')
    def getIndexValue(self, mimetype='text/plain'):
        """ an accessor method used for indexing the field's value
            XXX: the implementation is mostly based on archetype's
            `FileField.getIndexable` and rather naive as all data gets
            loaded into memory if a suitable transform was found.
            this should probably use `plone.transforms` in the future """
        field = self.getPrimaryField()
        source = field.getContentType(self)
        transforms = getToolByName(self, 'portal_transforms')

        if transforms._findPath(source, mimetype) is None:
            return ''
        value = str(field.get(self))
        filename = field.getFilename(self)
        try:
            return str(transforms.convertTo(
                mimetype,
                value,
                mimetype=source,
                filename=filename))
        except (ConflictError, KeyboardInterrupt):
            raise
        except:
            getLogger(__name__).exception(
                'exception while trying to convert '
                'blob contents to "text/plain" for %r', self)

    security.declareProtected(ModifyPortalContent, 'setFilename')
    def setFilename(self, value, **kw):
        field = self.getField('file')
        field.getUnwrapped(self).filename = value

    security.declareProtected(ModifyPortalContent, 'setOriginFilename')
    def setOriginFilename(self, value, **kw):
        """Overrides the filename with the given value.
        It keeps the old extenstion.
        """
        # file_delete contains '' if a file is uploaded, otherwise 'nochange'
        if self.REQUEST.form.get('file_delete', 'nochange') != 'nochange':
            return

        filename = self.getFilename()
        if not value or not filename:
            return

        if isinstance(value, unicode):
            value = value.encode('utf-8')

        self.setFilename('{0}{1}'.format(value, path.splitext(filename)[1]))

    security.declareProtected(View, 'getOriginFilename')
    def getOriginFilename(self):
        """Gets the filename without extension
        """
        filename = self.getFilename()
        if not filename:
            return None
        return path.splitext(filename)[0]

    security.declarePublic('getIcon')
    def getIcon(self, relative_to_portal=0):
        """Calculate the icon using the mime type of the file
        """

        field = self.getField('file')
        if not field or not self.get_size():
            # field is empty
            return BaseContent.getIcon(self, relative_to_portal)

        contenttype = field.getContentType(self)
        contenttype_major = contenttype and contenttype.split('/')[0] or ''

        mtr = getToolByName(self, 'mimetypes_registry', None)
        utool = getToolByName(self, 'portal_url')

        mimetypeitem = None
        try:
            mimetypeitem = mtr.lookup(contenttype)
        except MimeTypeException, msg:
            LOG = logging.getLogger('ATCT')
            LOG.error('MimeTypeException for %s. Error is: %s' % (
                self.absolute_url(), str(msg)))
        if not mimetypeitem:
            icon = None
        else:
            icon = mimetypeitem[0].icon_path

        if not icon:
            if contenttype in ICONMAP:
                icon = quote(ICONMAP[contenttype])
            elif contenttype_major in ICONMAP:
                icon = quote(ICONMAP[contenttype_major])
            else:
                return BaseContent.getIcon(self, relative_to_portal)

        if relative_to_portal:
            return icon
        else:
            # Relative to REQUEST['BASEPATH1']
            res = utool(relative=1) + '/' + icon
            while res[:1] == '/':
                res = res[1:]
            return res

    security.declarePublic('is_image')
    def is_image(self):
        file_ = self.getFile()
        return is_image(file_.getContentType())

    security.declareProtected(View, 'getField')
    def getField(self, key, *args, **kwargs):
        if key == 'image':
            key = 'file'
        return super(File, self).getField(key, *args, **kwargs)

atapi.registerType(File, PROJECTNAME)



import unittest as unittest
import doctest
from plone.testing import layered
from ftw.file.testing import FTW_FILE_INTEGRATION_TESTING


def test_suite():
    suite = unittest.TestSuite()
    suite.addTests([
        layered(doctest.DocFileSuite('file.txt'),
                layer=FTW_FILE_INTEGRATION_TESTING),
    ])
    return suite



from PIL import Image
from plone import api
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from zope.component import getUtility


def redirect_to_download_by_default(context):
    """Returns True if the default view of the file (context) should
    redirect to the download view.
    """

    request = context.REQUEST

    border_was_force_disabled = request.get('disable_border')
    if border_was_force_disabled:
        del request.other['disable_border']

    border_was_force_enabled = request.get('enable_border')
    if border_was_force_enabled:
        del request.other['enable_border']

    registry = getUtility(IRegistry)
    disable_download_redirect = registry.get('ftw.file.disable_download_redirect', False)

    try:
        if disable_download_redirect:
            return False
        plone_view = context.restrictedTraverse('@@plone')
        is_border_visible = plone_view.showEditableBorder()
        return not is_border_visible

    finally:
        if border_was_force_disabled:
            request.other['disable_border'] = border_was_force_disabled
        if border_was_force_enabled:
            request.other['enable_border'] = border_was_force_enabled


def is_image(mimetype):
    Image.init()
    open_handlers = Image.OPEN.keys()
    extensions = []
    for key, value in Image.EXTENSION.items():
        if value in open_handlers:
            extensions.append(key.strip('.'))

    mr = getToolByName(api.portal.get(), 'mimetypes_registry')
    mimetypes = mr.lookup(mimetype)
    if not mimetypes:
        return False  # unknown mimetype

    mime_extensions = mimetypes[0].extensions
    for ext in mime_extensions:
        if ext in extensions:
            return True
    for glob in mr.lookup(mimetype)[0].globs:
        if glob.strip("*.") in extensions:
            return True
    return False


class FileMetadata(object):
    """Handles some metadata of an object.

    The FileMetadata-object provides some properties and functions
    to get different pre-configured metadata of a file-object.
    """
    def __init__(self, context):
        self.context = context

    @property
    def show_author(self):
        """True when the file author should be displayed.

        False: if visitor is anonymous and siteproperty
               'allowAnonymousViewAbout' is False

        True:  if visitor is not anonymous
        True:  if visitor is anonymous and siteproperty
               'allowAnonymousViewAbout' is True
        """
        site_props = getToolByName(
            self.context, 'portal_properties').site_properties
        mt = getToolByName(self.context, 'portal_membership')

        if not site_props.getProperty('allowAnonymousViewAbout', False) \
                and mt.isAnonymousUser():
            return False
        return True

    @property
    def author(self):
        """Returns a dict with id, name and url of the author

        If the author has a fullname it takes it as the name
        If the autor has no fullname or if he have been deleted
        then it takes the userid as the name
        """
        pm = getToolByName(self.context, 'portal_membership')
        userid = self.context.Creator()
        member = pm.getMemberById(userid)
        if member:
            return dict(id=userid,
                        name=member.getProperty('fullname') or userid,
                        url=self.context.portal_url() + '/author/' + userid)
        return dict(id=userid,
                    name=userid,
                    url='')

    @property
    def document_date(self):
        """Returns the localized documentDate.
        """
        date = self.context.getDocumentDate()
        return self.context.toLocalizedTime(date)

    @property
    def modified_date(self):
        """Returns the localized modified-date.
        """
        modified = self.context.modified()
        return self.context.toLocalizedTime(modified, long_format=True)

    def get_image_tag(
            self, fieldname, width=None, height=None, direction="down"):
        """Returns the scaled image-tag if an image exists.
        """
        if not self.context.is_image():
            return None
        scale = self.context.restrictedTraverse('@@images')
        img = scale.scale(
            fieldname=fieldname,
            width=width,
            height=height,
            direction=direction)
        if img:
            return img.tag()
        return None

    @property
    def can_edit(self):
        """Returns true if the logged in user has
        modify-permission on the context.
        """
        mtool = getToolByName(self.context, 'portal_membership')
        return mtool.checkPermission('Modify portal content', self.context)



from ftw.builder import Builder
from ftw.builder import create
from ftw.file.testing import FTW_FILE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest import skipUnless
from unittest import TestCase
from zope.component import getMultiAdapter
import os.path


HAS_ACTIVITY_1 = False
try:
    from ftw.activity.interfaces import IActivityRepresentation
    HAS_ACTIVITY_1 = True
except ImportError:
    HAS_ACTIVITY_1 = False


@skipUnless(HAS_ACTIVITY_1, 'Detected ftw.activity 2.0')
class TestActivityRepresentation(TestCase):
    layer = FTW_FILE_FUNCTIONAL_TESTING

    def setUp(self):
        setRoles(self.layer['portal'], TEST_USER_ID, ['Contributor'])

    @browsing
    def test_renders_preview_for_images(self, browser):
        file_ = create(
            Builder('file')
            .titled('An important file')
            .attach_file_containing(self.asset('transparent.gif'),
                                    'transparent.gif'))
        self.render_representation(browser, file_)
        self.assertEqual('An important file',
                         browser.css('.title').first.text)

        self.assertTrue(browser.css('.colorboxLink'))
        self.assertFalse(browser.css('.file-download a'))

    @browsing
    def test_renders_download_link_for_files(self, browser):
        file_ = create(Builder('file')
                       .titled('An important file')
                       .with_dummy_content())
        self.render_representation(browser, file_)
        self.assertEqual('An important file',
                         browser.css('.title').first.text)

        self.assertFalse(browser.css('.colorboxLink'))
        self.assertTrue(browser.css('.file-download a'))

    def render_representation(self, browser, file_):
        repr = getMultiAdapter((file_, self.layer['request']),
                               IActivityRepresentation)
        self.assertTrue(repr.visible(),
                        'The activity representation is invisible.')
        html = repr.render()
        browser.open_html(html)

    def asset(self, filename):
        path = os.path.join(os.path.dirname(__file__),
                            'assets',
                            filename)
        with open(path, 'r') as fh:
            return fh.read()



from Products.statusmessages.interfaces import IStatusMessage
from ftw.file.events.events import FileDownloadedEvent
from ftw.file.imaging import ImagingMixin
from ftw.file.interfaces import IFtwFileField
from OFS.Image import File
from PIL.Image import ANTIALIAS
from plone.app.blob.field import FileField as BlobFileField
from plone.app.blob.download import handleIfModifiedSince
from plone.app.blob.download import handleRequestRange
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from urllib import quote
from webdav.common import rfc1123_date
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.event import notify
from zope.interface import implements

from ftw.file import fileMessageFactory as _

import logging


class FileField(BlobFileField, ImagingMixin):
    implements(IFtwFileField)

    _properties = BlobFileField._properties.copy()
    _properties.update({
        'pil_quality': 88,
        'pil_resize_algo': ANTIALIAS,
        'content_class': File,

    })

    def index_html(self, instance, REQUEST=None, RESPONSE=None,
                   charset='utf-8', disposition='inline'):
        """Kicks download.
        Writes data including file name and content type to RESPONSE
        """

        if REQUEST is None:
            REQUEST = instance.REQUEST

        if RESPONSE is None:
            RESPONSE = REQUEST.RESPONSE

        RESPONSE.setHeader('Last-Modified', rfc1123_date(instance._p_mtime))
        RESPONSE.setHeader('Content-Type', self.getContentType(instance))
        RESPONSE.setHeader('Accept-Ranges', 'bytes')

        if handleIfModifiedSince(instance, REQUEST, RESPONSE):
            return ''

        scan_redirect = self.scan_on_download(instance, REQUEST, RESPONSE)
        if scan_redirect:
            return scan_redirect

        length = self.get_size(instance)
        RESPONSE.setHeader('Content-Length', length)

        filename = self.getFilename(instance)
        if filename is not None:
            if isinstance(filename, unicode):
                filename = filename.encode(charset, errors="ignore")

            user_agent = REQUEST.get('HTTP_USER_AGENT', '')
            # Microsoft browsers disposition has to be formatted differently
            disposition_default = '{}; filename="{}"; filename={}*=UTF-8'.format(disposition, filename, filename)
            disposition_microsoft = '{}; filename={}; filename*=UTF-8\'\'{}'.format(disposition, filename, quote(filename))
            # use disposition_default by default
            disposition = disposition_default

            if any(key in user_agent for key in ['MSIE', 'WOW64', 'Edge']):
                # Set different dispositon if the user_agent
                # indicates download by a microsoft browser
                disposition = disposition_microsoft

            RESPONSE.setHeader("Content-disposition", disposition)

        request_range = handleRequestRange(instance, length, REQUEST, RESPONSE)

        # Notify file downloads, but do not notify range requests
        if not ('start' in request_range and request_range['start'] > 0):
            portal_state = getMultiAdapter((instance, instance.REQUEST),
                                           name='plone_portal_state')
            if not portal_state.anonymous():
                registry = getUtility(IRegistry)
                user_ids = registry['ftw.file.filesettings.user_ids']
                if portal_state.member().id \
                        and not portal_state.member().id in user_ids:
                    notify(FileDownloadedEvent(instance, filename))

        return self.get(instance).getIterator(**request_range)

    # TODO - do we need e.g. .... security.declareProtected(View, 'download') ?
    def scan_on_download(self, instance, REQUEST, RESPONSE):
        """ For preempting download of the file with a virus scan """
        try:
            from collective.clamav.validator import scanStream
            cclogger = logging.getLogger('collective.clamav.downloads')
            blobfile = instance.getFile().getBlob().open()
            result = scanStream(blobfile)
            if result:
                # result = 'virus_name FOUND'
                virus_name = result.replace(' FOUND', '')
                msgid = _(
                    u"download_not_possible",
                    default=u"Download not possible because the file contains a virus (${name}).",
                    mapping={u"name": virus_name}
                )
                tx_tool = getToolByName(instance, 'translation_service')
                IStatusMessage(REQUEST).addStatusMessage(tx_tool.translate(msgid), type='error')
                cclogger.warning((
                    "Download failed, file is virus-infected. ({}) filename: {}"
                ).format(virus_name, instance.getFilename()))
                file_view = instance.absolute_url() + "/file_view"
                return RESPONSE.redirect(file_view)
        except ImportError:
            pass

        # Scanning passed - or not available
        return None



from ftw.builder import Builder
from ftw.builder import create
from ftw.file.testing import FTW_FILE_FUNCTIONAL_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from Products.TinyMCE.adapters.interfaces.JSONDetails import IJSONDetails
from unittest import TestCase
import json


class TestJsondetails(TestCase):

    layer = FTW_FILE_FUNCTIONAL_TESTING

    def setUp(self):
        setRoles(self.layer['portal'], TEST_USER_ID, ['Manager'])
        self.file = create(Builder('file'))

    def test_jsondetails(self):
        details = IJSONDetails(self.file).getDetails()
        details = json.loads(details)
        self.assertEquals('file', details['title'])



from Products.CMFCore.utils import getToolByName
from ftw.file.testing import FTW_FILE_INTEGRATION_TESTING
from plone.app.testing import TEST_USER_NAME, TEST_USER_ID
from plone.app.testing import login, setRoles
from unittest import TestCase
import os


class TestFileName(TestCase):

    layer = FTW_FILE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        path = "%s/assets/testfile.NonsenseExtension" % (
            os.path.split(__file__)[0])
        self.file_ = open(path, 'r')
        self.mtr = getToolByName(self.portal, 'mimetypes_registry')
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

    def test_custom_icon(self):
        self.mtr.manage_addMimeType('customimage type',
                                    ['image/x-custom-image'],
                                    ['NonsenseExtension'],
                                    'customicon.png')
        plonefile = self.portal.get(self.portal.invokeFactory('File', 'myfile',
                                                              file=self.file_))
        self.assertEqual('plone/customicon.png', plonefile.getIcon())

    def test_default_icon(self):
        self.mtr.manage_addMimeType('customimage type',
                                    ['image/x-custom-image'],
                                    ['NonsenseExtension'],
                                    '')
        plonefile = self.portal.get(self.portal.invokeFactory('File', 'myfile',
                                                              file=self.file_))
        self.assertEqual('plone/image.png', plonefile.getIcon())

    def test_contenttype_not_found(self):
        # To test the behavior of our function when a contenttype is not found
        # in mimetyperegistry we add a mimetype o everything is indexed and
        # created correctly.
        self.mtr.manage_addMimeType('customimage type',
                                    ['image/x-custom-image'],
                                    ['NonsenseExtension'],
                                    '')
        plonefile = self.portal.get(self.portal.invokeFactory('File', 'myfile',
                                                              file=self.file_))
        # and then we remove it again so we won't find it in the registry.
        self.mtr.manage_delObjects(['image/x-custom-image'])

        self.assertIn(plonefile.getIcon(), (
                'plone/image_icon.gif',  # Plone <= 4.3.2
                'plone/image_icon.png',  # Plone >= 4.3.3
                ))



from ftw.builder import create
from ftw.builder import Builder
from ftw.file.testing import FTW_FILE_FUNCTIONAL_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest import TestCase
from ftw.testbrowser import browsing
from StringIO import StringIO
from PIL import Image


class TestFieldFunctions(TestCase):

    layer = FTW_FILE_FUNCTIONAL_TESTING

    def setUp(self):
        setRoles(self.layer['portal'], TEST_USER_ID, ['Manager'])
        self.file = create(Builder('file').attach_asset("testimage.jpg"))
        self.file.getPrimaryField().createScales(self.file)

    def test_available_sizes(self):
        expected = set(['mini', 'thumb', 'large',
                       'listing', 'tile', 'preview', 'icon'])
        sizes = self.file.getPrimaryField().getAvailableSizes(self.file).keys()
        sizes = set(sizes)
        self.assertTrue(expected.issubset(sizes),
                        "The available sizes don't include default scales.")

    def test_get_size(self):
        size = self.file.getPrimaryField().getSize(self.file, 'mini')
        self.assertEqual((200, 164), size, "Scale Sizes didn't match")

        size = self.file.getPrimaryField().getSize(self.file, 'preview')
        self.assertEqual((400, 328), size, "Scale Sizes didn't match")

    def test_get_scale(self):
        scale = self.file.getPrimaryField().getScale(self.file, 'mini')
        self.assertEqual('file_mini', scale.__name__)
        self.assertEqual(200, scale.width)
        self.assertEqual(164, scale.height)

    @browsing
    def test_traversion(self, browser):
        page = browser.login().visit(self.file, view="@@images/file/mini")
        image = Image.open(StringIO(page.contents))
        self.assertEqual((200, 164), image.size)

    @browsing
    def test__other_traversion(self, browser):
        page = browser.login().visit(self.file, view="file_mini")
        image = Image.open(StringIO(page.contents))
        self.assertEqual((200, 164), image.size)

    @browsing
    def test_traversion_equality(self, browser):
        page = browser.login().visit(self.file, view="file_mini")
        page2 = browser.login().visit(self.file, view="@@images/file/mini")
        self.assertEqual(page.contents,
                         page2.contents,
                         "The Two Image traversel methods aren't equal")



from setuptools import setup, find_packages
import os

version = '1.17.1'

tests_require = [
    'ftw.activity',
    'ftw.builder',
    'ftw.testbrowser >= 1.22',
    'ftw.testing',
    'plone.app.testing',
    'plone.mocktestcase',
    'pyquery',
    'collective.clamav[test] < 3.0a',
    ]

setup(name='ftw.file',
      version=version,
      description="A file content type for gov usecases",
      long_description=open("README.rst").read() + "\n" + \
          open(os.path.join("docs", "HISTORY.txt")).read(),

      # Get more strings from
      # http://www.python.org/pypi?%3Aaction=list_classifiers

      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.3',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],

      keywords='plone file contenttype',
      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      url='https://github.com/4teamwork/ftw.file',
      license='GPL2',

      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw'],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
        'Plone',
        'setuptools',
        'ftw.journal',
        'plone.app.registry',
        'ftw.upgrade>=1.14.3',
        'ftw.calendarwidget',
        'ftw.profilehook',
        'Pillow',
        'ftw.colorbox',
        'plone.api',
        'plone.batching',
        'plone.restapi < 7.0',
        'jsonschema < 3.0',
        # -*- Extra requirements: -*-
        ],
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),

      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from unittest import TestCase
from ftw.builder import create
from ftw.builder import Builder
from ftw.file.testing import FTW_FILE_FUNCTIONAL_TESTING
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from ftw.testbrowser import browsing
from zope.annotation.interfaces import IAnnotations
import os


class TestOutputfilter(TestCase):

    layer = FTW_FILE_FUNCTIONAL_TESTING

    def setUp(self):
        setRoles(self.layer['portal'], TEST_USER_ID, ['Manager'])

        self.obj = create(Builder('file').attach_file_containing(
                          self.asset('transparent.gif'), 'transparent.gif'))

    @browsing
    def test_outputfilter_scaled(self, browser):
        page = browser.login().visit(self.obj)
        annos = IAnnotations(self.obj)
        scales = annos.items()[1][1].keys()
        for scale in scales:
            if isinstance(scale, str):
                scale_uid = scale
        image = page.css('.fileListing img')[1].node
        self.assertEqual("http://nohost/plone/file/@@images/" + scale_uid + ".jpeg",
                         image.attrib['src'])

    def asset(self, filename):
        path = os.path.join(os.path.dirname(__file__),
                            'assets',
                            filename)
        with open(path, 'r') as fh:
            return fh.read()



from Acquisition import aq_inner
from plone.app.layout.viewlets import content
from plone.app.layout.viewlets.content import ContentHistoryView
from plone.batching.batch import Batch
from Products.CMFCore.utils import _checkPermission
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.CMFPlone.utils import log
from Products.CMFPlone import PloneMessageFactory as _
import AccessControl
import logging


class ContentHistoryViewlet(content.ContentHistoryViewlet):
    """ History viewlet for ftw.file
    """

    def show_viewlet(self):
        """ Permission Access previous versions is required"""
        if _checkPermission('CMFEditions: Access previous versions',
                            self.context):
            return True
        return False

    def workflowHistory(self, complete=False):
        """Return workflow history of this context.

        Taken from plone_scripts/getWorkflowHistory.py

        Do not check for 'Request review'
        """
        context = aq_inner(self.context)
        # check if the current user has the proper permissions
        if not _checkPermission('CMFEditions: Access previous versions',
                                context):
            return []

        workflow = getToolByName(context, 'portal_workflow')
        membership = getToolByName(context, 'portal_membership')

        review_history = []

        try:
            # get total history
            # Change SecurityManager
            _old_security_manager = AccessControl.getSecurityManager()
            _new_user = AccessControl.SecurityManagement.SpecialUsers.system
            AccessControl.SecurityManagement.newSecurityManager(
                context.REQUEST,
                _new_user)

            try:
                review_history = workflow.getInfoFor(context, 'review_history')
            except:
                AccessControl.SecurityManagement.setSecurityManager(
                    _old_security_manager)
                raise
            else:
                AccessControl.SecurityManagement.setSecurityManager(
                    _old_security_manager)

            if not complete:
                # filter out automatic transitions.
                review_history = [r for r in review_history if r['action']]
            else:
                review_history = list(review_history)

            portal_type = context.portal_type
            anon = _(u'label_anonymous_user', default=u'Anonymous User')

            for r in review_history:
                r['type'] = 'workflow'
                r['transition_title'] = workflow.getTitleForTransitionOnType(
                    r['action'], portal_type)
                actorid = r['actor']
                r['actorid'] = actorid
                if actorid is None:
                    # action performed by an anonymous user
                    r['actor'] = {'username': anon, 'fullname': anon}
                    r['actor_home'] = ''
                else:
                    r['actor'] = membership.getMemberInfo(actorid)
                    if r['actor'] is not None:
                        r['actor_home'] = \
                            self.navigation_root_url + '/author/' + actorid
                    else:
                        # member info is not available
                        # the user was probably deleted
                        r['actor_home'] = ''
            review_history.reverse()

        except WorkflowException:
            log('plone.app.layout.viewlets.content: '
                '%s has no associated workflow' % context.absolute_url(),
                severity=logging.DEBUG)

        return review_history

    @property
    def batch_size(self):
        return 10

    def history_batch(self):
        if 'b_start' in self.context.REQUEST.form:
            b_start = self.context.REQUEST.form['b_start']
        else:
            b_start = 0

        history = ContentHistoryView(self.context, self.context.REQUEST).fullHistory()
        if not history:
            return False

        batch = Batch(history, size=self.batch_size, start=b_start)
        return batch



from zope.interface import implements
from ftw.file.interfaces import IFileDownloadedEvent


class FileDownloadedEvent(object):
    """Event to notify when a file is downloaded"""
    implements(IFileDownloadedEvent)

    def __init__(self, obj, filename):
        self.object = obj
        self.filename = filename



from ftw.upgrade import UpgradeStep


class AddOptionToProtectFilesFromDeletion(UpgradeStep):
    """Add option to protect files from deletion.
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.builder import Builder, create
from ftw.file.tests import FunctionalTestCase
from ftw.testbrowser import browsing
import transaction


class TestAdvancedEdit(FunctionalTestCase):

    @browsing
    def test_no_advanced_edit(self, browser):
        self.grant('Contributor', 'Editor')
        browser.login().open(self.portal.absolute_url() + '/createObject?type_name=File')
        browser.css('fieldset')
        fieldsets = browser.css('fieldset')
        self.assertEqual(len(fieldsets), 0, "We found Fieldsets. We didn't expect any'")

    @browsing
    def test_advanced_edit(self, browser):
        self.grant('Reviewer')
        browser.login().open(self.portal.absolute_url() + '/createObject?type_name=File')
        fieldsets = browser.css('fieldset')
        self.assertEqual(
            ['Default', 'Categorization', 'Dates', 'Creators', 'Settings'],
            fieldsets.css('legend').text
        )

    @browsing
    def test_edit_date_fields(self, browser):
        self.grant('Contributor')
        file = create(Builder('file'))

        # By default, a user having the role "Contributor" does not have the permission
        # to edit the date fields.
        browser.login().open(file, view='edit')
        fieldsets = browser.css('fieldset')
        self.assertEqual(len(fieldsets), 0, "We found some fieldsets although we did not expect any.")

        # Grant the permission and make sure that the user is now able to edit the
        # date fields.
        file.manage_permission('ftw.file: Edit date fields', roles=['Contributor'], acquire=False)
        transaction.commit()
        browser.login().open(file, view='edit')
        fieldsets = browser.css('fieldset')
        self.assertEqual(
            ['Default', 'Dates'],
            fieldsets.css('legend').text
        )
